# coding: utf-8
'''
Created on 06.03.2015

@author: v.popov
'''

import json
from uuid import uuid4
from java.sql import Timestamp
from java.util import Date

from qt._qt_orm import QtiThemeCursor, QtiQuestionCursor, QtiDestructorCursor,\
    QtiVariantParamsCursor, QtiVariantAnswerCursor

from ru.curs.celesta.showcase.utils import XMLJSONConverter
from qt.functions import getAttributeValue,\
    createTestingVariantAssign, generateVariant
from nci.functions.persons import getPersonIdBySid
from nci import generalSettings
try:
    from ru.curs.showcase.core.jython import JythonDTO
    from ru.curs.showcase.core.selector import ResultSelectorData
    from ru.beta2.extra.gwt.ui.selector.api import DataRecord
    from ru.curs.showcase.app.api import UserMessage, MessageType
except:
    from ru.curs.celesta.showcase import JythonDTO, ResultSelectorData, DataRecord, UserMessage, MessageType
from medstaff._medstaff_orm import student_eor_tasksCursor


def cardData(context, main=None, add=None, filterinfo=None, session=None, elementId=None):
    u'''Данные для карточки.'''
    ss = json.loads(session)['sessioncontext']
    sid = ss['sid']
    personId = getPersonIdBySid(context, sid)
    
    package_id = ss['related']['gridContext']['currentRecordId']
    
    qti_theme = QtiThemeCursor(context)
    qti_question = QtiQuestionCursor(context)
    qti_destructor = QtiDestructorCursor(context)
    qti_theme.setRange('PackageID',package_id)
    count = 0
    errorText = ''
    emptyFlag = False
    for theme in qti_theme.iterate():
        qti_question.setRange('ThemeID',theme.ThemeID)
        count += qti_question.count()
        questionList = []
        for question in qti_question.iterate():
            qti_destructor.setRange('QuestionID',question.QuestionID)
            if qti_destructor.count() == 0:
                questionList.append(str(question.QuestionOrder))
        if questionList:
            errorText += u'В теме "%s" вопросы №%s не содержат вариантов ответа. '%(theme.ThemeName,','.join(questionList))
            emptyFlag = True
    if count == 0:
        emptyFlag = True
        errorText = u'БТЗ не содержит вопросов'
        
    qti_theme.close()
    qti_question.close()
    qti_destructor.close()        
    
    modeId = 'test'
    variant_id = unicode(uuid4())

    # Структура данных
    xformsdata = {'schema': {  
                    '@xmlns':'',
                    'data': {  
                        'mode_name':'',
                        'mode_id':modeId,
                        'variant_id': variant_id,
                        'package_id': package_id,
                        'person_id': personId,
                        '@empty': 'true' if emptyFlag else 'false',
                        'errorText': errorText}}}

    # Первоначальная xformssettings
    xformssettings = {  
        'properties': {  
            'event': [  
                {'@name': 'single_click',
                 '@linkId': '1',
                 'action': {
                     "#sorted": [
                        {'main_context': 'current'},                
                        {'datapanel': {  
                            '@type': 'current',
                            '@tab': 'current',
                            'element': [  
                                {'@id': 'studentQtiGrid',
                                 'add_context': 'refresh'},
                                {'@id': elementId,
                                 'add_context': 'hide'}]}},
                                {"client": {
                                    "activity":[
                                        {"@id": "debugBTZ",
                                         "@name": "goToTest",
                                         "add_context": u"&mode=%s&package=%s&param=%s" % (modeId,package_id,variant_id)}]}}]}}]}}
    
    return JythonDTO(XMLJSONConverter.jsonToXml(json.dumps(xformsdata)), XMLJSONConverter.jsonToXml(json.dumps(xformssettings)))


def cardSave(context, main=None, add=None, filterinfo=None, session=None, elementId=None, xformsdata=None):
    u'''Сохранение карточки. '''
    session = json.loads(session)
    sid = session['sessioncontext']['sid']
    data = json.loads(xformsdata)['schema']['data']
    package_id = data['package_id']
    variant_id = data['variant_id']
    person_id = data['person_id']    
    
    main_jsn = json.loads(main)
    cycle_id = main_jsn['cycle_id']
    certificate = main_jsn['certificate']
#     mode = data['mode_id']    
#     setAccessPermission(context, sid, mode, variant_id)
    # Создание информации о тесте
    
    params = {'NoRandom': 0,
              'TimeTest': generalSettings.getGlobalSettings(context, 'TEST_TIME'),
              'QuantityQuestions': generalSettings.getGlobalSettings(context, 'QUANTITY_QUESTIONS'),
              'Percent3': 60,
              'Percent4': 70,
              'Percent5': 80,
              'NoProtocol': 0,
              'NoSelfTesting': 1}
    
    createTestingVariantAssign(context, variant_id, package_id, sid, person_id, params, cycle_id)
    # Генерация теста
    generateVariant(context, package_id, variant_id, debugFlag=False)
    
    QtiVariantParams = QtiVariantParamsCursor(context)
    timeTest = int(getAttributeValue(QtiVariantParams, variant_id, 'TimeTest'))
    QtiVariantAnswer = QtiVariantAnswerCursor(context)
    QtiVariantAnswer.VariantAnswerID = unicode(uuid4())
    QtiVariantAnswer.VariantID = variant_id
    timeBegin = Date()
    QtiVariantAnswer.TimeBegin = timeBegin
    QtiVariantAnswer.TimeEnd = Timestamp(timeBegin.getTime() + timeTest * 60 * 1000)
    QtiVariantAnswer.insert()
    
    variantAnswerCur = student_eor_tasksCursor(context)
#     if variantAnswerCur.tryGet(person_id, certificate, cycle_id, package_id) and  variantAnswerCur.attempts_count > 0:
#         variantAnswerCur.attempts_count -= 1
#         variantAnswerCur.variant_id = variant_id
#         variantAnswerCur.update()
